(*  
    : Title : IOTuGames.m
    : Release Date : 27.07.2005   

*)

(* :Context: IOTuGames` *)

(* :Summary:	
    This package provides some graphical extensions to the package TuGames.
*)			

(* : Author : 
     Holger Ingmar Meinhardt
     Department of Economics University of Karlsruhe
     hme@vwl3.wiwi.uni-karlsruhe.de
*)
(*
   : Mathematica Version : 3.0, 4.0, 4.1, 4.2, 5.0, 5.1
*)


(*  :Sources:
    Mathematica Graphics by Tom Wickham-Jones, Springer-Telos, 1994.

*)

(*:Requirements: 

   The Mathematica package ExtendGraphics`View3D by Tom Wickham-Jones.
   Moreover, the package needs MathGL3d from Jens-Peer Kuska, cddml and some Mathematica packages by Komei Fukuda.
   Moreover qhull and geomview are required to use OGGL format for displaying the graphics and rotating raw
   3D graphics. 
   http://phong.informatik.uni-leipzig.de/~kuska/mview3d.html (MathGL3D)
   http://www.cs.mcgill.ca/~fukuda (cddml, PolytopeSkeleton.m, IOPolyhedra.m, UnfoldPolytope.m)
   http://www.geom.umn.edu/software/qhull
   http://www.geomview.org  

*)



BeginPackage["IOTuGames`"]


ShowKernelCatcher::usage = 
"ShowKernelCatcher[dirname, filename, game, options] plots the interlocking of the imputation and 
lower or proper reasonable set as a skeleton. For an explanation of the input parameters call the function PlotCore3D."

InterLockingImpReasSet::usage = 
"InterLockingImpReasSet[dirname, filename, game, options] plots the interlocking of the imputation and 
reasonable set as a skeleton. For an explanation of the input parameters call the function PlotCore3D."

NewDirectory::usage =
"NewDirectory[dirname] set the working directory from the default directory /usr/local/bin to directory name 
'dirname'."

PlotCore3D::usage = 
"PlotCore3D[dirname, filename, game, options] plots the core of the game. The parameter `dirname` is the working directory 
where the qhull input file `filename` has been saved. As an additional parameter the game must be specified."

SaveVertices::usage = 
"SaveVertices[dirname, filename, vertices] saves the vertices of the core in `filename` of the working directory `dirname`."

ShowNucleolus::usage = "ShowNucleolus[game, options]"

ShowKernel::usage = "ShowKernel[game, options]"

ShowShapley::usage = "ShowShapley[game, options]"

SkelCore::usage = 
"SkelCore[dirname, filename, game, options] depicts the skeleton of the core. For an explanation 
of the input parameters call the function PlotCore3D."

SkelImputationSet::usage = 
"SkelImputationSet[dirname, filename, game, options] plots the imputation set as a skeleton. For an explanation 
of the input parameters call the function PlotCore3D."

SkelReasonableSet::usage = 
"SkelReasonableSet[dirname, filename, game, options] plots the reasonable set of a game as a skeleton. For an explanation 
of the input parameters call the function PlotCore3D."

SkelLowerSet::usage = 
"SkelLowerSet[dirname, filename, game, options] plots the lower set of a game as a skeleton. For an explanation 
of the input parameters call the function PlotCore3D."

SkelStrongEpsCore::usage = 
"SkelStrongEpsCore[dirname, filename, game,options] depicts the skeleton of the strong epsilon core. For an explanation 
of the input parameters call the function PlotCore3D."

AnimationKernelProperty::usage
"AnimationKernelProperty[dirname,filename,options] visualize the relationship between the kernel solution and the strong epsilon core."

VertexFacetsIncidences::usage = 
"VertexFacetsIncidences[game,options] depicts the relationship between the vertices and the facets of the core in matrix form."

CriticalValue::usage =
"Is an option for the functions ShwoKernelCatcher[] and SkelStrongEpsCore[].
 The default value of the option can be changed by CriticalValue->{integer}.
 Admissible integer number are 1,2,3,4,5, and 6. This selects a critical
 value for the strong epsilon core."


SetOptions[Graphics3D,ViewPoint -> {1.714, 2.478, 1.540}]
defopts = Sort[Flatten[{OOGLFile -> False, OutputDirname -> False, InternalViewPoint->True, ViewKernelSol -> False, KernelCoord -> {}, ViewShapleySol -> False, ShapleyCoord -> {}, ViewNucleolusSol-> False, NucleolusCoord->{}, Options[Graphics3D]}]]



Options[GeneratePolyhedron] = Options[defopts] 
Options[PlotCore3D] = Options[defopts] 

defopts1 = Sort[Flatten[{ShowSkeleton -> True, OOGLFile -> False, OutputDirname -> False , InternalViewPoint->True, ViewKernelSol -> False, KernelCoord -> {}, ViewShapleySol -> False, ShapleyCoord -> {}, ViewNucleolusSol-> False, NucleolusCoord->{}, MvOutput -> False, WithProperContribution -> False, Options[Graphics3D]}]]


Options[SkelCore] = Options[defopts1] 
Options[SkelImputationSet] = Options[defopts1] 
Options[SkelReasonableSet] = Options[defopts1] 
Options[InterLockingImpReasSet] = Options[defopts1]
Options[ShowKernel] = Options[defopts1]
Options[ShowNucleolus] = Options[defopts1]
Options[ShowShapley] = Options[defopts1]
Options[CheckNucEqKernel] = Options[defopts]
Options[GenShapleyGraphPoint] = Options[defopts1]
Options[GenKernelGraphPoint] = Options[defopts1]
Options[GenKernelGraphLine] = Options[defopts1]
Options[GenNucleolusGraphPoint] = Options[defopts1]

Options[VertexFacetsIncidences] = Options[defopts1]

defoptskel = Sort[Flatten[{ShowStrongEpsCore -> True, ShowCore -> False, EpsValues -> {}, CriticalValue -> {1}, Options[defopts1]}]]
Options[SkelStrongEpsCore] = Sort[Flatten[{KcOutput -> False, Options[defoptskel]}]]
Options[SubCallStrCore] = Sort[Options[defoptskel]]
Options[AnimationKernelProperty] = Sort[Flatten[{UpperCriticalVal ->{1/2}, LowerCriticalVal -> {0},StepSize -> {-(1/4)},Options[defoptskel]}]]

defopts3 = Sort[Flatten[{DisplayAsMatrix -> False, AsIncidenceMatrix -> True, AsDigits -> False, WithFaceNumber -> False, NumberOfVertices -> False}]]
Options[ShowKernelCatcher] = Sort[Flatten[{ShowCore->False, ShowLowerSet->False, ShowUpperSet-> False, ShowStrongEpsCore -> False, MvOutput -> False, EpsValues -> {}, CriticalValue -> {6}, WithProperContribution -> False, Options[defopts1], Options[defopts3]}]]



Begin["`Private`"]

Needs["coop`CooperativeGames`"];
Needs["TuGames`"];


Which[$MachineName === "xxx", Needs["View3D`"];,
        $MachineName === "xxx", Needs["View3D`"];,
        $MachineName === "xxx-e202", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx-e101", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx-e102", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx-e103", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx-e000", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx-e001", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx-e002", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx-e003", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx", Needs["ExtendGraphics`View3D`"];,
        True, Needs["View3D`"];
       ]



<<PolytopeSkeleton.m
<<IOPolyhedra.m
<< UnfoldPolytope.m
Needs["Utilities`FilterOptions`"]


PlotCore3D[dirname_String, filename_String, game_, opts___?OptionQ] := 
  Module[{norm,normgame,vertlist,fig},
    writeoogl = OOGLFile /. {opts} /. Options[GeneratePolyhedron]; 
    writedir = OutputDirname /. {opts} /. Options[GeneratePolyhedron];
    norm = OneNormalization[game];
    normgame = DefineGame[T,norm]; 
    vertlist = CddGmpVerticesCore[normgame];
    vertlist = CheckVert[normgame,vertlist];
    fig = If[Length[vertlist]<=2,GenCoreGraphLine[vertlist,opts],GeneratePolyhedron[dirname, filename, N[vertlist], opts]];
    Show[fig]
    ]

CheckVert[game_,vertices_List]:=
  Module[{crQ,poc},
       crQ = BelongToCoreQ[game, vertices];
       poc=Position[crQ,True];
       Extract[vertices,poc]

 ]



SaveVertices[dirname_String, filename_String, vertices_] := 
  Module[{datei, savepath, tofile, sp1, sp2, sp3, m1, d1, str1},
    datei = ToString[filename];
    savepath = ToFileName[$HomeDirectory, dirname];
    tofile = ToFileName[savepath, datei];
    {m1, d1} = Dimensions[vertices];
    sp1 = ToString["# "];
    sp2 = ToString[d1];
    sp3 = ToString["-d input"];
    str1 = StringJoin[sp1, sp2, sp3];
    Put[str1, d1, m1, vertices, tofile]
    ]


GeneratePolyhedron[dirname_String, filename_String, vertices_, 
    opts___?OptionQ ] := 
  Module[{writeoogl, writedir,proc1, proc2, res1, res2, tofile, dirset, dirold, dirnew, filename1,
       filename2, filename3,polyhedron,gr,filopts},
    writeoogl = OOGLFile /. {opts} /. Options[GeneratePolyhedron]; 
    writedir = OutputDirname /. {opts} /. Options[GeneratePolyhedron];
    filopts = FilterOptions[Graphics3D, opts]; 
    dirold = Directory[];
    ResetDirectory[];
    dirset = ToFileName[$HomeDirectory, dirname];
    dirnew = SetDirectory[dirset];
    filename1 = ToString[filename] <> ToString[".asc"];
    filename2 = ToString[filename] <> ToString[".txt"];
    filename3 = ToString[filename] <> ToString[".txt"] <> ToString[".asc"];
    SaveVertices[dirname, filename1, vertices];
    proc1 = OpenRead["!qh2sed " <> ToString[ filename1]];
    res1 = Read[proc1, String];
    If[ res1 === EndOfFile, Return[$Failed]];
    Close[proc1];
    proc2 = OpenRead["!qhull2mama " <> ToString[ filename2]];
    res2 = Read[proc2, String];
    If[ res2 === EndOfFile, Return[$Failed]];
    Close[proc2];
    tofile = ToFileName[dirnew, filename3];
    polyhedron = Get[tofile];
    gr = Graphics3D[polyhedron, filopts];
    If[writeoogl == True, WriteFileToOOGL[dirnew,filename,gr]];
    ResetDirectory[];
    SetDirectory[dirold];
    If[writedir == False, gr, {gr,dirnew}]
    ]


WriteFileToOOGL[dirname_String, filename_String, graphicObj_] := 
 Module[{fileoogl,tofile1},
    fileoogl = ToString[filename] <> ToString[".oogl"];
    tofile1 = ToFileName[dirname, fileoogl];
    WriteOOGL[tofile1,graphicObj];
]



NewDirectory[dirname_String]:= 
  Module[{dirold,dirset,dirnew},
    dirold = Directory[];
    ResetDirectory[];
    dirset = ToFileName[$HomeDirectory, dirname];
    dirnew = SetDirectory[dirset];
    ResetDirectory[];
    SetDirectory[dirold];
    dirnew
]



ShowKernel[game_,opts___?OptionQ] := 
 Module[{normQ,kercoord, filopts,kersol,l1},
    kercoord = KernelCoord /. {opts} /. Options[ShowKernel];
    normQ = Which[Depth[kercoord] == 2,Apply[Plus,kercoord],
                  Depth[kercoord] == 3,Apply[Plus,#]&/@kercoord,
                True,1];
    kercoord = Which[Depth[kercoord] == 2, 
                       Which[normQ == 1, kercoord, 
                         True, kercoord/normQ],
                    Depth[kercoord] == 3, 
                       Which[(normQ[[1]] == 1 && normQ[[2]] == 1),kercoord,
                         True,kercoord/normQ[[1]]],
               True, kercoord];
    l1 = Depth[kercoord];
    filopts = FilterOptions[Graphics3D, opts]; 
    Which[(Length[kercoord]!=0 && l1==2)==True, GenKernelGraphPoint[kercoord,opts],
          (Length[kercoord]!=0 && l1==3) == True, GenKernelGraphLine[kercoord,opts],
          True, kersol = Kernel[game];GenKernelGraphPoint[kersol,opts]
    ]
(*
    reddim = Drop[kersol,1];
    kerpts = Point[reddim];
    Graphics3D[{ RGBColor[1,0,0], PointSize[0.025], kerpts }]
*)
] 


GenKernelGraphPoint[payoff_,opts___?OptionQ]:= 
   Module[{reddim,kerpts,showskl},
    showskl = ShowSkeleton /. {opts} /. Options[GenKernelGraphPoint]; 
    reddim = If[Length[payoff]==4,Drop[payoff,1],payoff];
    kerpts = Point[reddim];
    Which[showskl == True, Graphics3D[{ RGBColor[1,0,0], PointSize[0.030], kerpts }],
      True, Return[kerpts]]
]

GenCoreGraphLine[payoff_List,opts___?OptionQ]:= GenKernelGraphLine[payoff,opts] 

GenKernelGraphLine[payoff_List,opts___?OptionQ]:= 
   Module[{reddim,kerseg},
    showskl = ShowSkeleton /. {opts} /. Options[GenKernelGraphLine];
    reddim = Drop[#,1]&/@ payoff;
    kerseg = Line[reddim];
    Which[showskl == True, Graphics3D[{ RGBColor[1,0,0], AbsoluteThickness[5], kerseg }],
       True, Return[kerseg]]
]



ShowShapley[game_,opts___?OptionQ] := 
    Module[{normQ,shapcoord,shapsol,rdim,l1,retval},
    shapcoord = ShapleyCoord /. {opts} /. Options[ShowShapley];
    normQ = Which[Length[shapcoord] != 0, Apply[Plus,shapcoord],True,1];
    shapcoord = Which[Length[shapcoord] != 0, Which[normQ == 1, shapcoord, True, shapcoord/normQ], True, shapcoord];
    l1 = Depth[shapcoord];
    retval=Which[(Length[shapcoord]!=0 && l1==2)==True,GenShapleyGraphPoint[shapcoord,opts],
    True,shapsol = NewShapley[game];GenShapleyGraphPoint[shapsol,opts]];
    Return[retval]
(*
    rdim = Drop[shapsol,1];
    shpts = Point[rdim];
    Graphics3D[{ RGBColor[1,1,0], PointSize[0.025], shpts }]
*)
]


GenShapleyGraphPoint[payoff_,opts___?OptionQ]:= 
   Module[{reddim,shpts,showskl},
    showskl = ShowSkeleton /. {opts} /. Options[GenShapleyGraphPoint]; 
    reddim = If[Length[payoff]==4,Drop[payoff,1],payoff];
    shpts = Point[reddim];
    Which[showskl === True, Graphics3D[{ RGBColor[1,1,0], PointSize[0.025], shpts }],
      True, Return[shpts]]
]


ShowNucleolus[game_,opts___?OptionQ] := 
    Module[{normQ,nuccoord,nucsol,rdim,l1},
    nuccoord = NucleolusCoord /. {opts} /. Options[ShowNucleolus];
    normQ = Which[Length[nuccoord] != 0, Apply[Plus, nuccoord], True, 1];
    nuccoord = Which[Length[nuccoord] != 0, Which[normQ == 1, nuccoord, True, nuccoord/normQ], True, nuccoord];
    l1 = Depth[nuccoord];
    Which[(Length[nuccoord]!=0 && l1==2)==True,GenNucleolusGraphPoint[nuccoord,opts],
    True,nucsol = ModifiedNucleolus[game];GenNucleolusGraphPoint[nucsol,opts]]
(*
    rdim = Drop[nucsol,1];
    nucpts = Point[rdim];
    Graphics3D[{ RGBColor[.5,.5,1], PointSize[0.025], nucpts }]
*)
]


GenNucleolusGraphPoint[payoff_,opts___?OptionQ]:= 
   Module[{reddim,nucpts,showskl},
    shownuc = ShowSkeleton /. {opts} /. Options[GenNucleolusGraphPoint]; 
    reddim = If[Length[payoff]==4,Drop[payoff,1],payoff];
    nucpts = Point[reddim];
    Which[shownuc == True, Graphics3D[{ RGBColor[.5,.5,.25], PointSize[0.025], nucpts }],
      True, Return[nucpts]]
]


AnimationKernelProperty[dirname_String, filename_String, game_, opts___?OptionQ] := 
  Module[{shepscr, shcr, shsk, epsv, crv, oogl, vwker, kcoord, vsh, shc, vnuc,
       nucc, uppval, lowval, stpsize, vpt},
    shepscr = ShowStrongEpsCore /. {opts} /. Options[AnimateKernelProp];
    shcr = ShowCore /. {opts} /. Options[AnimateKernelProp];
    shsk = ShowSkeleton /. {opts} /. Options[AnimateKernelProp];
    crv = CriticalValue /. {opts} /. Options[AnimateKernelProp];
    oogl = OOGLFile /. {opts} /. Options[AnimateKernelProp];
    vwker = ViewKernelSol /. {opts} /. Options[AnimateKernelProp];
    kcoord = KernelCoord /. {opts} /. Options[AnimateKernelProp];
    vsh = ViewShapleySol /. {opts} /. Options[AnimateKernelProp];
    shc = ShapleyCoord /. {opts} /. Options[AnimateKernelProp];
    vnuc = ViewNucleolusSol /. {opts} /. Options[AnimateKernelProp];
    nucc = NucleolusCoord /. {opts} /. Options[AnimateKernelProp];
    uppval = UpperCriticalVal /. {opts} /. Options[AnimateKernelProp];
    lowval = LowerCriticalVal -> {0} /. {opts} /. Options[AnimateKernelProp];
    stpsize = StepSize /. {opts} /. Options[AnimateKernelProp];
    vpt = ViewPoint /. {opts} /. Options[AnimateKernelProp];
    detlow = 
      Which[CoreQ[game] === False, FirstCriticalVal[game][[1, 2]], True, 
        lowval[[1]]];
    Do[SkelStrongEpsCore[dirname, filename, game, EpsValues -> {t}, 
        ShowCore -> shcr, ViewPoint -> vpt, ViewKernelSol -> vwker, 
        KernelCoord -> kcoord, ViewNucleolusSol -> vnuc, 
        NucleolusCoord -> nucc, ViewShapleySol -> vsh, 
        ShapleyCoord -> shc], {t, uppval[[1]], detlow[[1]], stpsize[[1]]}]
    ]



SkelStrongEpsCore[dirname_String, filename_String, game_, opts___?OptionQ] :=
    Module[{critval,norm,normgame,strgskl,kcout,orgval,nonskel, mvplt, skel, shcr},
     Off[Show::gtype];
     strgskl = ShowStrongEpsCore /. {opts} /. Options[SkelStrongEpsCore];
     shcr = ShowCore /. {opts} /. Options[SkelStrongEpsCore];
     kcout = KcOutput /. {opts} /. Options[SkelStrongEpsCore];
     critval = CriticalValue /. {opts} /. Options[SkelStrongEpsCore];
     mvplt = MvOutput /. {opts} /. Options[SkelStrongEpsCore];
     norm = OneNormalization[game]; 
     normgame = DefineGame[T,norm];
(* EpsGame[] changes the coalitional values. orgval stores the original values.  *)
     orgval = Rationalize[v[#] & /@ Subsets[T]];
    skel = Which[strgskl === True,
                 Which[(kcout && (!mvplt)) === True, MvSkelEpsCore[dirname,filename,normgame, opts, CriticalValue -> critval], 
                       ((!kcout) && (!mvplt)) === True, SubCallStrCore[dirname,filename,normgame, opts, CriticalValue -> critval],
                       ((!kcout) && mvplt) === True, 
                             Which[shcr === True, Print["here"];SubCallStrCore[dirname,filename,normgame, opts, CriticalValue -> critval],
                             True, MvSkelEpsCore[dirname,filename,normgame, opts, CriticalValue -> critval]],
                 True, MvSkelEpsCore[dirname,filename,normgame, opts, CriticalValue -> critval] 
	        ],
             True, MvSkelEpsCore[dirname,filename,normgame, opts, CriticalValue -> critval] 
       ];
     On[Show::gtype];
(*   Reconstruction of the original game. *)
     DefineGame[T,orgval];
     Return[skel]
  ]


SubCallStrCore[dirname_String, filename_String, game_,opts___?OptionQ] := 
 Module[{filename1,sklcore,shcore,critval,crq,epsval,epslist,eps,newgame,sklst,filopts,gra1,gra2,kergr,shapgr,viewker,showskl},
(*     dispvar = DisplayFunction /. {opts} /. Options[SkelStrongEpsCore]; *)
     showskl = ShowSkeleton /. {opts} /. Options[SkelStrongEpsCore];
     epsval = EpsValues /. {opts} /. Options[SkelStrongEpsCore]; 
     critval = CriticalValue /. {opts} /. Options[SkelStrongEpsCore]; 
     shcore = ShowCore /. {opts} /. Options[SkelStrongEpsCore];
     viewker = ViewKernelSol /. {opts} /. Options[SkelStrongEpsCore];
(*     viewnuc = ViewNucleolusSol  /. {opts} /. Options[SkelStrongEpsCore]; *)
     viewshap = ViewShapleySol /. {opts} /. Options[SkelStrongEpsCore];
     strgskl = ShowStrongEpsCore /. {opts} /. Options[SkelStrongEpsCore];
     filename1 = ToString[filename] <> ToString["org"];
     crq = CoreQ[game];
     critval = Which[{critval}==={1},Which[crq ===False,{6},True,{1}],True,Which[crq ===False,{6},True,critval]];
     sklcore = SkelCore[dirname,filename1,game,opts,ShowSkeleton->False];
     epslist = Which[(Length[epsval] == 0) == True, ConstEpsList[game],
                      True,epsval];
     Global`eps = Which[critval[[1]] === 1, Part[epslist,1], critval[[1]]===2,Part[epslist,2],
                 critval[[1]]===3, Part[epslist,3],critval[[1]]===4, Part[epslist,4],critval[[1]]===5, Part[epslist,5],critval[[1]]===6, 
                                   Part[epslist,6],True, Part[epslist,6]];
     newgame = EpsValue[game, Global`eps];
     sklst = SkelCore[dirname,filename,newgame,opts,ShowSkeleton->False];
     filopts = FilterOptions[Graphics3D, opts];
     Which[((!viewker) && (!viewshap) && (!shcore)) === True, gra2 = Graphics3D[sklst,filopts]; Show[gra2,PlotLabel -> StringJoin["eps=",ToString[N[Global`eps]]]], 
(*FFT *)            ((!viewker) && (!viewshap) && shcore) === True, gra1 = Graphics3D[sklcore]; gra2 = Graphics3D[sklst];
                               Show[gra1,gra2,filopts,  PlotLabel -> StringJoin["eps=",ToString[N[Global`eps]]]],
(*FTT *)             ((!viewker) && viewshap && shcore) === True, gra1 = Graphics3D[sklcore]; gra2 = Graphics3D[sklst];
                               Show[gra1,gra2,filopts, PlotLabel -> StringJoin["eps=",ToString[N[Global`eps]]]],
(*FTF *)             ((!viewker) && viewshap && (!shcore)) === True,  gra2 = Graphics3D[sklst];
                               Show[gra2,filopts, PlotLabel -> StringJoin["eps=",ToString[N[Global`eps]]]],
(*TFF *)             (viewker && (!viewshap) && (!shcore)) === True, kergr = CheckNucEqKernel[game,opts]; gra2 = Graphics3D[sklst];
                               If[Length[kergr]==1,Show[kergr,gra2,filopts, PlotLabel -> StringJoin["eps=",ToString[N[Global`eps]]]],
                                       Show[kergr[[1]],kergr[[2]],gra2,filopts, PlotLabel -> StringJoin["eps=",ToString[N[Global`eps]]]]],
(*TFT *)             (viewker && (!viewshap) && shcore) === True, kergr = CheckNucEqKernel[game,opts]; gra1 = Graphics3D[sklcore]; 
                                                gra2 = Graphics3D[sklst];
                               If[Length[kergr]==1,Show[kergr,gra1,gra2,filopts, PlotLabel -> StringJoin["eps=",ToString[N[Global`eps]]]],
                                       Show[kergr[[1]],kergr[[2]],gra1,gra2,filopts, PlotLabel -> StringJoin["eps=",ToString[N[Global`eps]]]]],
(*TTF *)             (viewker && viewshap && (!shcore)) === True, kergr = CheckNucEqKernel[game,opts];shapgr = ShowShapley[game,opts];
                                           gra2 = Graphics3D[sklst];
                               If[Length[kergr]==1,Show[kergr,shapgr,gra2,filopts, PlotLabel -> StringJoin["eps=",ToString[N[Global`eps]]]],
                                       Show[kergr[[1]],kergr[[2]],shapgr,gra2,filopts,  PlotLabel -> StringJoin["eps=",ToString[N[Global`eps]]]]],
(*TTT*)             True, kergr = CheckNucEqKernel[game,opts]; shapgr = ShowShapley[game,opts]; gra1 = Graphics3D[sklcore]; gra2 = Graphics3D[sklst];
                               If[Length[kergr]==1,Show[kergr,shapgr,gra1,gra2,filopts, PlotLabel -> StringJoin["eps=",ToString[N[Global`eps]]]],
                                  Show[kergr[[1]],kergr[[2]],shapgr,gra1,gra2,filopts, PlotLabel -> StringJoin["eps=",ToString[N[Global`eps]]]]
                                 ]
            ]
  ]



MvSkelEpsCore[dirname_String, filename_String, game_,opts___?OptionQ] := 
 Module[{filename1,sklcore,shcore,critval,epsval,epslist,eps,newgame,sklst,filopts,gra1,gra2,kergr,shapgr,viewker,showskl},
(*     showskl = ShowSkeleton /. {opts} /. Options[SkelStrongEpsCore];*)
     epsval = EpsValues /. {opts} /. Options[SkelStrongEpsCore]; 
     critval = CriticalValue /. {opts} /. Options[SkelStrongEpsCore]; 
     shcore = ShowCore /. {opts} /. Options[SkelStrongEpsCore];
     filename1 = ToString[filename] <> ToString["org"];
     crq = CoreQ[game];
     critval = Which[{critval}==={1},Which[crq ===False,{6},True,{1}],True,Which[crq ===False,{6},True,critval]];
     sklcore = SkelCore[dirname,filename1,game,opts,ShowSkeleton->False];
     epslist = Which[(Length[epsval] == 0) == True,ConstEpsList[game],
                      True,epsval];
     Global`eps = Which[critval[[1]] === 1, Part[epslist,1], critval[[1]]===2,Part[epslist,2],
                 critval[[1]]=== 3, Part[epslist,3],critval[[1]]=== 4, Part[epslist,4],critval[[1]]=== 5, 
                  Part[epslist,5],critval[[1]] === 6, Part[epslist,6],True, Part[epslist,6]];
     newgame = EpsValue[game, Global`eps];
     sklst = SkelCore[dirname,filename1,newgame,opts,ShowSkeleton->False];
     If[shcore ===True, Return[{sklcore,sklst}],Return[{Take[sklcore,-8],sklst}]]
  ]



ConstEpsList[game_] := 
    Module[{seccrit, thirdcrit,fourth,starcrit,starcrit2,starcrit3,epsval2,epsval3,epsval4,epsstar,epsstar2,epsstar3,epslist},
       seccrit = SecondCriticalVal[game];
       epsval2 = Global`eps2 /. seccrit;
       thirdcrit = ThirdCriticalVal[game];
       epsval3 = Global`eps3 /.thirdcrit;
       fourth = FourthCriticalVal[game];
       epsval4 = Global`eps4 /.fourth;
       starcrit = StarCriticalVal[game];
       epsstar = Global`stareps /.starcrit;
       starcrit2 = SecondStarCriticalVal[game];
       epsstar2 = Global`dstareps /.starcrit2;
       starcrit3 = ThirdStarCriticalVal[game];
       epsstar3 = Global`thstareps /.starcrit3;
       epslist= {epsval2,epsval3,epsval4,epsstar,epsstar2,epsstar3};
       epslist

 ] 


SkelCore[dirname_String, filename_String, game_, opts___?OptionQ] := 
   Module[{norm,normgame,vertices, newext, extreml, extlist, extpts, impv, coreelQ, post, linearity,ecdlist, eadlist, eadl2, icdlist, iadlist, inedata,figure,amat,bvec,vlist0,uvp1,m0,d0,endofeadl, redead1,redead2, neweadlist,dirnew,skl,fileskloogl,tofile2,writeoogl,writedir,viewpt,viewker,viewnuc,showskl,filopts,kergr,shapgr,nucgr},
      norm = OneNormalization[game]; 
      normgame = DefineGame[T,norm];
      {{extreml, linearity}, ecdlist, eadlist, icdlist, iadlist, inedata} = CddGmpPlotCore[normgame];
       Off[Dot::dotsh];
       Off[Set::setraw];
       Off[Graphics3D::nlist3];
       writeoogl = OOGLFile /. {opts} /. Options[SkelCore]; 
       writedir = OutputDirname /. {opts} /. Options[SkelCore];
       viewpt = InternalViewPoint /. {opts} /. Options[SkelCore];
       viewker = ViewKernelSol /. {opts} /. Options[SkelCore];
       viewnuc = ViewNucleolusSol /. {opts} /. Options[SkelCore];
       viewshap = ViewShapleySol /. {opts} /. Options[SkelCore];
       showskl = ShowSkeleton /. {opts} /. Options[SkelCore];
       extpts = Drop[extreml,-Length[T]]; 
       impv = Take[extreml,-Length[T]];
       {m0,d0}= Dimensions[extreml];
       lng = m0-Length[T];
       eadl2 = eadlist;
       {coreelQ,eadlist} = ExtractCoreEl[normgame,extpts,eadl2];
       extlist = FlattenAt[Append[coreelQ,impv],lng+1];
       vertices = Drop[extlist,-Length[T]];
       Which[d0 == 4, newext = AdjustVert[normgame,extlist];, 
             d0 == 3, newext = AdjustVert[normgame,extlist];,
             True, Print["Dimension is not correct", d0];BreakCal[d0]];
       newext = N[newext];
       dirnew = NewDirectory[dirname];
       amat=-Transpose[Drop[Transpose[inedata],1]];
       bvec=Transpose[inedata][[1]];
       vlist0 = If[d0 == 4,Map[Drop[#, 1]&, ToExpression[newext]],newext];
       uvp1 = circle[Pi/11, 50, 20];
       endofeadl = Take[eadlist,-Length[T]]; 
       redead1 = DeleteCases[#,lng+1]&/@ Drop[eadlist,-Length[T]];
       redead2 = ConstructEadList[redead1,lng+2, m0+1];
       neweadlist = FlattenAt[Append[redead2,endofeadl],lng+1];
       skl = skel3D[uvp1,vlist0, ecdlist, neweadlist, amat, bvec];
       If[writeoogl == True, WriteSkelToOOGL[dirnew,filename,skl]];
       filopts = FilterOptions[Graphics3D, opts];
       Which[showskl == True,
       Which[viewpt==True,
         Show[skl,Boxed->False, 
 		ViewPoint -> getMmaViewPoint[uvp1,skel3D[uvp1,vlist0, ecdlist, eadlist, amat, bvec]],
 		SphericalRegion->True],
             (viewker || viewshap) === False, Show[skl,filopts],
             (viewker === True) && (viewshap === False), kergr = CheckNucEqKernel[normgame,opts]; 
                         If[Length[kergr]==1,Show[skl,kergr,filopts],Show[skl,kergr[[1]],kergr[[2]],filopts]],
             (viewker === False) && (viewshap === True), shapgr = ShowShapley[normgame,opts]; Show[skl,shapgr,filopts],
             True, kergr = CheckNucEqKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts];
                         If[Length[kergr]==1,Show[skl,kergr,shapgr,filopts],Show[skl,kergr[[1]],kergr[[2]],shapgr,filopts]]
        ],True,Return[VisibleSkeleton[vlist0, ecdlist, neweadlist,{amat, bvec}, uvp1]]];
       On[Dot::dotsh];
       On[Set::setraw];
       On[Graphics3D::nlist3];

]

CheckNucEqKernel[game_,opts___?OptionQ]:=
    Module[{viewnuc,kersol,nucsol,kergra,nucgra},
     viewnuc = ViewNucleolusSol  /. {opts} /. Options[CheckNucEqKernel];
     kersol = ShowKernel[game,opts,ShowSkeleton->False];
     If[viewnuc == True, nucsol = ShowNucleolus[game,opts,ShowSkeleton->False];,nucsol = kersol];
     If[kersol===nucsol,kergra = ShowKernel[game,opts],kergra={ShowKernel[game,opts],ShowNucleolus[game,opts]}];
     kergra
  ]




ConstructEadList[ead_List,ln_Integer,UpperBound_Integer]:=
  Module[{constead,k},
      constead = DeleteCases[#,ln] &/@ ead; 
      k = ln+1;
      If[k != UpperBound, ConstructEadList[constead,k,UpperBound],constead]
]


ExtractCoreEl[game_,vertices_,eadlist_List]:= 
   Module[{crQ,allcQ},
     crQ = BelongToCoreQ[game, vertices];
     allcQ = Apply[And,crQ];
     If[allcQ == True, {vertices,eadlist}, SubCallCore[vertices,eadlist,crQ]]

]

SubCallCore[vertices_,eadlist_List,coreQ_List]:=
   Module[{poc,nov,vert,eadl},
     poc=Position[coreQ,True];
     nov=Position[coreQ,False];    
     vert = ReplacePart[vertices,{NAN},nov];
     eadl = RecDeleteVert[eadlist,Flatten[nov]];
     {vert,eadl}

]


RecDeleteVert[eadlist_List, nonvert_List] := Module[{neweal, newnonv},
    fi = First[nonvert];
    neweal = DeleteVert[eadlist, fi];
    newnonv = Drop[nonvert, 1];
    If[Length[newnonv] != 0, RecDeleteVert[neweal, newnonv], neweal]
    ]

DeleteVert[eadliste_,num_Integer]:= 
  Module[{},
    DeleteCases[#,num]&/@ eadliste
    ]


ReduceDim[vertices_List]:= Module[{lng,smallest,largest,newvert,vertli,li},
     lng = Map[Last, vertices];
     smallest = Min[lng]; 
     largest = Max[lng];
     newvert = Map[{fun[(Last[#] - min)/(max - min)], Take[#, 3]} &, vertices];
     vertli = DeleteCases[#, fun[_]] & /@ newvert;
     li = Outer[List, Table[i, {i, Length[vertli]}]];
     FlattenAt[vertli, li]
     
]


VertexFacetsIncidences[game_,opts___?OptionQ]:=
  Module[{norm,normgame,disasmat, asmat, facenb, nbvert, nbv, extreml, linearity,ecdlist, eadlist, icdlist, iadlist, inedata, m1, d1,n1, extpts, redicd, cele, vli, newicd, matinc, premat,eqnumb},
      disasmat = DisplayAsMatrix /. {opts} /. Options[VertexFacetsIncidences]; 
      asmat = AsIncidenceMatrix /. {opts} /. Options[VertexFacetsIncidences]; 
      facenb = WithFaceNumber /. {opts} /. Options[VertexFacetsIncidences]; 
      nbvert = NumberOfVertices /. {opts} /. Options[VertexFacetsIncidences]; 
      norm = OneNormalization[game]; 
      normgame = DefineGame[T,norm];
      {{extreml, linearity}, ecdlist, eadlist, icdlist, iadlist, inedata} = CddGmpPlotCore[normgame];
      {m1, d1} = Dimensions[extreml];
       n1 = m1-d1;
       extpts = Drop[extreml,-Length[T]];
       redicd = Drop[icdlist, -2]; (* Value two is not correct for T != 4 *)
       listicd = Table[i,{i,Length[redicd]}];
       (*       celeQ = BelongToCoreQ[normgame,extpts]; *)
       {cele, icdlist} = ExtractCoreEl[normgame,extpts,icdlist];
       cele = DeleteCases[cele,{NAN}]; 
       vli = Table[i, {i, Length[cele]}];
       (*      extvli = If[nbvert == True, Table[i, {i, 0, n1+1}], Table[i, {i, 0, n1}]];*)
       extvli = Table[i, {i, 0, Length[cele]}];
       newicd = DropingSuperfluousVertexInc[redicd,Length[cele],m1,opts];
       nbv = Prepend[Length[#] &/@ newicd,Length[cele]+1];
       Which[asmat == True,
             matinc = GeneratingIncidenceMatrix[#,vli,opts] &/@ newicd;
             matinc = If[facenb == True,  MapThread[Prepend,{matinc,listicd}], matinc];  
             premat = If[facenb == True,  Prepend[matinc,extvli], Prepend[matinc,vli]];
             appmat = If[(facenb && nbvert) == True,  MapThread[Append, {premat,nbv}], premat];
         Which[disasmat == False, matinc, 
              True, If[(facenb && nbvert) == True, MatrixForm[appmat], MatrixForm[premat]]],
       True, newicd]
]


DropingSuperfluousVertexInc[icd_List, l_Integer, UpperBound_Integer,opts___?OptionQ] :=
     Module[{ticd,k},
    ticd = DeleteCases[#, l+1] & /@ icd;
    k = l+1;
    If[k != UpperBound, DropingSuperfluousVertexInc[ticd, k, UpperBound], Return[ticd]]
    ]


GeneratingIncidenceMatrix[newicd_List, allvli_List,opts___?OptionQ] := 
  Module[{asdig,compicd, rwicd, rwcomp, setmat},
    compicd = Complement[allvli, newicd];
    rwicd = Outer[List, newicd];
    rwcomp = Outer[List, compicd];
    setmat = SetMatrix[allvli,rwicd,rwcomp, opts];
(*    setone = ReplacePart[allvli, 1, rwicd];
    setzero = ReplacePart[setone, "0" , rwcomp]; *) 
    setmat
    ]

SetMatrix[allext_List, rmicd_List, rmcomp_List, opts___?OptionQ] := 
    Module[{asdig, setone, setval, setstar},
    asdig = AsDigits /. {opts} /. Options[VertexFacetsIncidences]; 
      Which[asdig == False,
                         setstar = ReplacePart[allext, StyleForm["*", FontWeight -> "Bold", FontSize -> 24, FontColor -> RGBColor[1, 0, 0]], rmicd];
                         setval =  ReplacePart[setstar, ".", rmcomp],
           True, setone = ReplacePart[allext, StyleForm[1, FontWeight -> "Bold", FontSize -> 14, FontColor -> RGBColor[1, 0, 0]], rmicd];
                 setval = ReplacePart[setone, 0, rmcomp]];
      Return[setval]
    ]


ShowKernelCatcher[dirname_String, filename_String, game_, opts___?OptionQ]:=
  Module[{norm,normgame,viewker,viewnuc,viewshap,mvout,shcr,strCr,shlws,shups,reass,filename1,filename2,filename3,filename4,kergr,nucgr,shapgr,sklX,sklR,sklLgrsX,grsR,grsL,grsC,takeC,filopts},
        viewker = ViewKernelSol /. {opts} /. Options[ShowKernelCatcher];
        viewshap = ViewShapleySol /. {opts} /. Options[ShowKernelCatcher];
        viewnuc = ViewNucleolusSol /. {opts} /. Options[ShowKernelCatcher];
        shups = ShowUpperSet  /. {opts} /. Options[ShowKernelCatcher];
        shcr = ShowCore /. {opts} /. Options[ShowKernelCatcher];
        strCr = ShowStrongEpsCore /. {opts} /. Options[ShowKernelCatcher];
        shlws = ShowLowerSet /. {opts} /. Options[ShowKernelCatcher];
(*        reasset = ShowReasonableSet /. {opts} /. Options[ShowKernelCatcher];*)
        mvout = MvOutput /. {opts} /. Options[ShowKernelCatcher];
        Which[shups===True,$REASSET="Upper Set",True,$REASSET="Reasonable Set"];
        norm = OneNormalization[game]; 
        normgame = DefineGame[T,norm];
        filename1 = ToString[filename] <> ToString["1"];
        filename2 = ToString[filename] <> ToString["2"];
        filename3 = ToString[filename] <> ToString["3"];
        filename4 = ToString[filename] <> ToString["4"];
	sklX = SkelImputationSet[dirname, filename1, normgame, ShowSkeleton -> False, opts];
	sklR = Which[shups===False, SkelReasonableSet[dirname, filename2, normgame, ShowSkeleton -> False, opts],
                     True, SkelReasonableSet[dirname, filename2, normgame, ShowSkeleton -> False, ProperContribution -> True, opts]];
	sklL = SkelLowerSet[dirname, filename3, normgame, ShowSkeleton -> False, opts];
        grsX = Graphics3D[{EdgeForm[{RGBColor[1,0,0], Thickness[0.02]}], sklX}];
        grsR = Graphics3D[{EdgeForm[{RGBColor[1,0,0], Thickness[0.02]}], sklR}];
        grsL = Graphics3D[{EdgeForm[{RGBColor[1,0,0], Thickness[0.02]}], sklL}];
        grsC = Which[((!shcr) && (!strCr))===True,{{},{}},
                     ((!shcr) && strCr) ===True,DrawCore3D[dirname, filename4, normgame, strCr, shcr,opts],
                     (shcr && (!strCr)) ===True,DrawCore3D[dirname, filename4, normgame, strCr,shcr, opts],
                     True, DrawCore3D[dirname, filename4, normgame, strCr, shcr, opts]];
(*    Which[Length[grsC]===1,grsC=grsC[[1]],True,{grsC1,grsC2}= {grsC[[1]],grsC[[2]]}] *)
        grsC = Flatten[grsC];
        filopts = FilterOptions[Graphics3D, opts];
    Which[mvout===True,
            Which[((!shcr) && (!shups) && (!shlws) && (!strCr))===True,Return[{grsX,grsR}],(* reasonable set*)
                  ((!shcr) && shups && (!shlws) && (!strCr))===True,Return[{grsX,grsR}],(* upper set*)
                  ((!shcr) && (!shups) && shlws && (!strCr))===True,Return[{grsX,grsR,grsL}],
                  ((!shcr) && shups && shlws && (!strCr))===True,Return[{grsX,grsR,grsL}],
(*                  (strCr && reass && (!shlws) && (!strCr))===True,Return[{grsX,grsC,grsR}],*)
                  (shcr && (!shups) && (!shlws) && (!strCr))===True,Return[Flatten[{grsX,grsC,grsR}]],
                  (shcr && shups && (!shlws) && (!strCr))===True,Return[Flatten[{grsX,grsC,grsR}]],
                  (shcr && (!shups) && shlws && (!strCr))===True,Return[Flatten[{grsX,grsC,grsL,grsR}]],
                  (shcr && shups && shlws && (!strCr))===True,Return[Flatten[{grsX,grsC,grsR,grsL}]],
                  (shcr && shups && shlws && strCr)===True,Return[Flatten[{grsX,grsC,grsR,grsL}]], (*length of grsC is 2*)
                  ((!shcr) && shups && shlws && strCr)===True,Return[Flatten[{grsX,grsC,grsR,grsL}]], (* upper set and length of grsC is 1*)
                  ((!shcr) && shups && (!shlws) && strCr)===True,Return[Flatten[{grsX,grsC,grsR,grsL}]], (* upper set and length of grsC is 1*)
                  ((!shcr) && (!shups) && shlws && strCr)===True,Return[Flatten[{grsX,grsC,grsR,grsL}]], (*reasonable set*)
                  ((!shcr) && (!shups) && (!shlws) && strCr)===True,Return[Flatten[{grsX,grsC,grsR,grsL}]], (*reasonable set*)
           True,Return[Flatten[{grsX,grsC,grsR,grsL}]]],
    True,
     Which[((!shcr) && (!shups) && (!shlws) && (!strCr))===True,Print["Plotting the Intersection of the ",$REASSET];Print["and the Imputation Set."];
         Which[((!viewker) && (!viewshap) && (!viewnuc)) === True, Show[grsX,grsR,filopts],
             (viewker && viewnuc && (!viewshap)) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsR,kergr,nucgr,filopts],
             (viewker && (!viewnuc) && (!viewshap)) === True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsR,kergr,filopts],
             (viewker && viewshap && (!viewnuc)) === True,kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,kergr,shapgr,filopts],
             ((!viewker) && viewshap && (!viewnuc)) === True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,shapgr,filopts],
           True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,kergr,shapgr,filopts]],
       ((!shcr) && shups && (!shlws) && (!strCr))===True,Print["Plotting the ",$REASSET];Print["and the Imputation Set."];
         Which[((!viewker) && (!viewshap) && (!viewnuc)) === True, Show[grsX,grsR,filopts],
             (viewker && (!viewshap) && viewnuc) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsR,kergr,nucgr,filopts],
             (viewker && (!viewnuc) && (!viewshap))  === True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsR,kergr,filopts],
             (viewker && viewshap && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,kergr,shapgr,filopts],
             ((!viewker) && (!viewnuc) && viewshap) === True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,shapgr,filopts],
           True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,kergr,shapgr,filopts]],
       ((!shcr) && (!shups) && shlws && (!strCr))===True,Print["Plotting the Lower and the Imputation Set."];
         Which[((!viewker) && (!viewshap) && (!viewnuc)) === True, Show[grsX,grsL,filopts],
             (viewker && (!viewshap) && viewnuc) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsL,kergr,nucgr,filopts],
             (viewker && (!viewshap) && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsL,kergr,filopts],
             ((!viewker) && viewshap && (!viewnuc)) === True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsL,shapgr,filopts],
             (viewker && viewshap && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsL,kergr,shapgr,filopts],
           True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsL,kergr,shapgr,filopts]],
       ((!shcr) && shups && shlws && (!strCr))===True,Print["Plotting the Lower, ",$REASSET]; Print["and the Imputation Set."];
         Which[((!viewker) && (!viewshap) && (!viewnuc)) === True, Show[grsX,grsR,grsL,filopts],
            (viewker && (!viewshap) && viewnuc) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsR,grsL,kergr,nucgr,filopts],
             (viewker && (!viewshap) && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsR,grsL,kergr,filopts],
            (viewker && viewshap && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,grsL,kergr,shapgr,filopts],
             (!(viewker) && viewshap && (!viewnuc)) === True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,grsL,shapgr,filopts],
           True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,grsL,kergr,shapgr,filopts]],
       ((!shcr) && shups && shlws && strCr) === True, Print["Plotting the Strong Epsilon Core, the Lower, ",$REASSET]; Print["and the Imputation Set."];
         Which[((!viewker) && (!viewshap) && (!viewnuc)) === True, Show[grsX,grsC,grsR,grsL,filopts],
       (viewker && (!viewshap) && viewnuc) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsC,grsR,grsL,kergr,nucgr,filopts],
       (viewker && (!viewshap) && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsC,grsR,grsL,kergr,filopts],
       (viewker && viewshap && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,grsR,grsL,kergr,shapgr,filopts],
       (!(viewker) && viewshap && (!viewnuc)) === True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,grsR,grsL,shapgr,filopts],
           True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,grsR,grsL,kergr,shapgr,filopts]],
       ((!shcr) && (!shups) && shlws && strCr) === True, Print["Plotting Plotting the Strong Epsilon Core, the Lower, ",$REASSET]; Print["and the Imputation Set."];
         Which[((!viewker) && (!viewshap) && (!viewnuc)) === True, Show[grsX,grsC,grsR,grsL,filopts],
       (viewker && (!viewshap) && viewnuc) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsC,grsR,grsL,kergr,nucgr,filopts],
       (viewker && (!viewshap) && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsC,grsR,grsL,kergr,filopts],
       (viewker && viewshap && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,grsR,grsL,kergr,shapgr,filopts],
       (!(viewker) && viewshap && (!viewnuc)) === True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,grsR,grsL,shapgr,filopts],
           True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,grsL,kergr,shapgr,filopts]],
       ((!shcr) && shups && (!shlws) && strCr) === True, Print["Plotting Plotting the Strong Epsilon Core, the Lower, ",$REASSET]; Print["and the Imputation Set."];
         Which[((!viewker) && (!viewshap) && (!viewnuc)) === True, Show[grsX,grsC,grsR,filopts],
       (viewker && (!viewshap) && viewnuc) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsC,grsR,kergr,nucgr,filopts],
       (viewker && (!viewshap) && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsC,grsR,kergr,filopts],
       (viewker && viewshap && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,grsR,kergr,shapgr,filopts],
       (!(viewker) && viewshap && (!viewnuc)) === True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,grsR,shapgr,filopts],
           True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,kergr,shapgr,filopts]],
       ((!shcr) && (!shups) && (!shlws) && strCr) === True, Print["Plotting Plotting the Strong Epsilon Core, the Lower, ",$REASSET]; Print["and the Imputation Set."];
         Which[((!viewker) && (!viewshap) && (!viewnuc)) === True, Show[grsX,grsR,grsC,filopts],
       (viewker && (!viewshap) && viewnuc) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsC,grsR,kergr,nucgr,filopts],
       (viewker && (!viewshap) && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsC,grsR,kergr,filopts],
       (viewker && viewshap && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,grsR,kergr,shapgr,filopts],
       (!(viewker) && viewshap && (!viewnuc)) === True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,grsR,shapgr,filopts],
           True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,kergr,shapgr,filopts]],
     (shcr && (!shlws) && (!shups) && (!strCr)) ===True,Print["Plotting the Core, ",$REASSET];Print["and the Imputation Set."];
        Which[((!viewker) && (!viewshap) && (!viewnuc)) === True, Show[grsX,grsC,grsR,filopts],
            (viewker && (!viewshap) && viewnuc) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsC,grsR,kergr,nucgr,filopts],
             (viewker && (!viewshap) && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsC,grsR,kergr,filopts],
            (viewker && viewshap && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,grsR,kergr,shapgr,filopts],
             ((!viewker) && viewshap && (!viewnuc))=== True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,grsR,shapgr,filopts],
           True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,grsR,kergr,shapgr,filopts]],
     (shcr && shlws && (!shups) && (!strCr)) ===True,Print["Plotting the Core, the Lower and the Imputation Set."];
        Which[((!viewker) && (!viewshap) && (!viewnuc)) === True, Show[grsX,grsC,grsL,filopts],
           (viewker  && (!viewshap) && viewnuc) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsC,grsL,kergr,nucgr,filopts],
             (viewker && (!viewshap) && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsC,grsL,kergr,filopts],
            (viewker && viewshap && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,grsL,kergr,shapgr,filopts],
             ((!viewker) && viewshap && (!viewnuc)) === True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,grsL,shapgr,filopts],
           True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,grsL,kergr,shapgr,filopts]],
     (shcr && shups && (!shlws) && (!strCr))===True, Print["Plotting the Core, ",$REASSET];Print["and the Imputation Set."];
        Which[(viewker || viewshap || viewnuc) == False, Show[grsX,grsR,grsC,filopts],
            (viewker && viewnuc && (!viewshap)) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsR,grsC,kergr,nucgr,filopts],
             (viewker && (!viewnuc) && (!viewshap)) === True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsR,grsC,kergr,filopts],
            (viewker && viewshap && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,grsC,kergr,shapgr,filopts],
             ((!viewker) && (!viewnuc) && viewshap) === True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,grsC,shapgr,filopts],
           True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,grsC,kergr,shapgr,filopts]],
     (shcr && shups && shlws && (!strCr))===True,  Print["Plotting the Core, the Lower, ",$REASSET];Print["and the Imputation Set."];
       Which[((!viewker) && (!viewshap) && (!viewnuc)) === True, Show[grsX,grsR,grsC,grsL,filopts],
       (viewker && (!viewshap) && viewnuc) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsR,grsC,grsL,kergr,nucgr,filopts],
             (viewker && (!viewshap) && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsR,grsC,grsL,kergr,filopts],
       (viewker && viewshap && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,grsC,grsL,kergr,shapgr,filopts],
             ((!viewker) && viewshap && (!viewnuc)) === True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,grsC,grsL,shapgr,filopts],
           True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsR,grsC,grsL,kergr,shapgr,filopts]],
     (shcr && (!shups) && (!shlws) && strCr)===True,  Print["Plotting the Strong Epsilon Core, the Core, ",$REASSET];Print["and the Imputation Set."];
        Which[((!viewker) && (!viewshap) && (!viewnuc)) === True, Show[grsX,grsC,filopts],
             (viewker && (!viewshap) && viewnuc) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsC,kergr,nucgr,filopts],
             (viewker && (!viewshap) && (!viewnuc))=== True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsC,kergr,filopts],
             (viewker && viewshap && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,kergr,shapgr,filopts],
             ((!viewker) && viewshap && (!viewnuc))=== True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,shapgr,filopts]],
     (shcr && shups && (!shlws) && strCr)===True,  Print["Plotting the Strong Epsilon Core, the Core, ",$REASSET];Print["and the Imputation Set."];
        Which[((!viewker) && (!viewshap) && (!viewnuc)) === True, Show[grsX,grsC,filopts],
             (viewker && (!viewshap) && viewnuc) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsC,kergr,nucgr,filopts],
             (viewker && (!viewshap) && (!viewnuc))=== True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsC,kergr,filopts],
             (viewker && viewshap && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,kergr,shapgr,filopts],
             ((!viewker) && viewshap && (!viewnuc))=== True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,shapgr,filopts]],
      (shcr && (!shups) && shlws && strCr)===True,  Print["Plotting the Strong Epsilon Core, the Core, the Lower Set"];Print[" and the Imputation Set."];
        Which[((!viewker) && (!viewshap) && (!viewnuc)) === True, Show[grsX,grsC,filopts],
             (viewker && (!viewshap) && viewnuc) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsC,kergr,nucgr,filopts],
             (viewker && (!viewshap) && (!viewnuc))=== True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsC,kergr,filopts],
             (viewker && viewshap && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,kergr,shapgr,filopts],
             ((!viewker) && viewshap && (!viewnuc))=== True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,shapgr,filopts],
           True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,kergr,shapgr,filopts]],
     (shcr && shups && shlws && strCr)===True,  Print["Plotting the Strong Epsilon Core, the Core, ",$REASSET];Print["the Lower Set and the Imputation Set."];
        Which[((!viewker) && (!viewshap) && (!viewnuc)) === True, Show[grsX,grsC,filopts],
             (viewker && (!viewshap) && viewnuc) === True, kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsC,kergr,nucgr,filopts],
             (viewker && (!viewshap) && (!viewnuc))=== True, kergr = ShowKernel[normgame,opts]; Show[grsX,grsC,kergr,filopts],
             (viewker && viewshap && (!viewnuc)) === True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,kergr,shapgr,filopts],
             ((!viewker) && viewshap && (!viewnuc))=== True, shapgr = ShowShapley[normgame,opts]; Show[grsX,grsC,shapgr,filopts]],
     True,Print["Plotting the Strong Epsilon Core, the Core, the Lower, ",$REASSET];Print["and the Imputation Set."];
           kergr = ShowKernel[normgame,opts];shapgr = ShowShapley[normgame,opts];Show[grsX,grsR,grsL,grsC,kergr,shapgr,filopts]
    ]
  ];
]




DrawCore3D[dirname_String, filename_String, game_,strCr_,shcr_,opts___?OptionQ]:=Module[{crq,crit,skellist,dimval,dirnew,graph1,grpah2,figlist1},
        crit = CriticalValue /. {opts} /. Options[ShowKernelCatcher];
        crq = Which[shcr === True, CoreQ[game], True, shcr];
        crit = Which[crit===1,Which[crq ===False,6,True,1],True,Which[crq ===False,6,True,crit]];
	skellist = SkelStrongEpsCore[dirname, filename, game, opts,
                         ShowStrongEpsCore -> strCr, ShowCore -> True, CriticalValue -> {crit},
                         ViewPoint -> {1.714, 2.478, 1.540}, KcOutput -> True, ViewKernelSol -> False, ViewShapleySol->False];
        skellist = If[Length[T]==3,AdjustSkelList[skellist],skellist];
        graph1 = skellist[[1]]; (* Core Skeleton *)
        graph2 = skellist[[2]]; (* Strong Core Skeleton *)
(*        figlist1 = Graphics3D[{EdgeForm[{RGBColor[1,0,0], Thickness[0.02]}], #}] &/@ {graph1,graph2};*)
        figlist1 = Graphics3D[#, Axes -> False, Boxed -> False, 
                   ViewPoint -> {1.714, 2.478, 1.540},
                   LightSources -> {{{0.335180, -1.1415, -0.76458}, 
                   RGBColor[1., 0., 0.]}, {{1.5922, 0., -0.681822}, 
                   RGBColor[0., 1., 0.]}, {{0.5276, -0.82099, -1.0235144}, 
                   RGBColor[0., 0., 1.]}, {{0., 0., 1.5}, RGBColor[0.25, 0.25, 0.25]}},
                   DisplayFunction -> Identity]&/@ {graph1,graph2};
        figlist1 = Which[(crq && (!strCr)) === True, Drop[figlist1,-1],
                         ((!crq) && strCr) === True,Drop[figlist1,1],
                         True, figlist1];
        Return[figlist1]
]



InterLockingImpReasSet[dirname_String, filename_String, game_, opts___?OptionQ]:=
  Module[{norm,normgame,viewker,viewnuc,viewshap,propcont,filename1,filename2,kergr,nucgr,shapgr,sklX,sklR,grsX,grsR,filopts},
        viewker = ViewKernelSol /. {opts} /. Options[InterLockingImpReasSet];
        viewshap = ViewShapleySol /. {opts} /. Options[InterLockingImpReasSet];
        viewnuc = ViewNucleolusSol /. {opts} /. Options[InterLockingImpReasSet];
        propcont =  WithProperContribution /. {opts} /. Options[InterLockingImpReasSet];
        norm = OneNormalization[game]; 
        normgame = DefineGame[T,norm];
        filename1 = ToString[filename] <> ToString["1"];
        filename2 = ToString[filename] <> ToString["2"];
	sklX = SkelImputationSet[dirname, filename1, normgame, ShowSkeleton -> False, opts];
	sklR = Which[propcont===False, SkelReasonableSet[dirname, filename2, normgame, ShowSkeleton -> False, opts],
                     True, SkelReasonableSet[dirname, filename2, normgame, ShowSkeleton -> False, ProperContribution -> True, opts]];
        grsX = Graphics3D[{EdgeForm[{RGBColor[1,0,0], Thickness[0.02]}], sklX}];
        grsR = Graphics3D[{EdgeForm[{RGBColor[1,0,0], Thickness[0.02]}], sklR}];
        filopts = FilterOptions[Graphics3D, opts];
        Which[(viewker || viewshap || viewnuc) == False, Show[grsX,grsR,filopts],
             (viewker == True) && (viewnuc == True), kergr = ShowKernel[normgame,opts]; nucgr = ShowNucleolus[normgame,opts]; Show[grsX,grsR,kergr,nucgr,filopts],
             (viewker == True) && (viewshap == False), kergr = ShowKernel[normgame,opts]; Show[grsX,grsR,kergr,filopts],
             (viewker == False) && (viewshap == True), shapgr = ShowShapley[normgame]; Show[grsX,grsR,shapgr,filopts],
           True, kergr = ShowKernel[normgame,opts]; shapgr = ShowShapley[normgame]; Show[grsX,grsR,kergr,shapgr,filopts]];
]



SkelImputationSet[dirname_String, filename_String, game_, opts___?OptionQ] := 
   Module[{norm,normgame, vertices, newext, extlist, linearity,ecdlist, eadlist, icdlist, iadlist, inedata,figure,amat,bvec,vlist0,uvp1,m0,d0,dirnew,skl,fileskloogl,tofile2},
      showskl = ShowSkeleton /. {opts} /. Options[SkelImputationSet]; 
      norm = OneNormalization[game]; 
      normgame = DefineGame[T,norm];
      {{extlist, linearity}, ecdlist, eadlist, icdlist, iadlist, inedata} = CddGmpPlotImputationSet[normgame];
      Off[Dot::dotsh];
      Off[Part::partw];
      Off[Line::list];
      {m0,d0}= Dimensions[extlist];
      vertices = extlist;
      newext = N[vertices];
      dirnew = NewDirectory[dirname];
      amat=-Transpose[Drop[Transpose[inedata],1]];
      bvec=Transpose[inedata][[1]];
      vlist0 = Map[Drop[#, 1]&, ToExpression[newext]];
      uvp1 = circle[Pi/11, 50, 20];
      endofeadl = Take[eadlist,-4];
      redead1 = DeleteCases[#,m0+1]&/@ Drop[eadlist,-4];
      redead2 = DeleteCases[#,m0+2]&/@ redead1;
      redead3 = DeleteCases[#,m0+3]&/@ redead2;
      redead4 = DeleteCases[#,m0+4]&/@ redead3;
(*      neweadlist = FlattenAt[Append[redead4,endofeadl],m0+1];*)
      skl = skel3D[uvp1,vlist0, ecdlist, redead4, amat, bvec];
      fileskloogl = ToString[filename] <> ToString[Skel] <> ToString[".oogl"];
      tofile2 = ToFileName[dirnew, fileskloogl];
      WriteOOGL[tofile2,skl];
      Which[showskl == True,
       Show[skl,Boxed->False, 
 		ViewPoint -> getMmaViewPoint[uvp1,skel3D[uvp1,vlist0, ecdlist, redead4, amat, bvec]],
 		SphericalRegion->True
 	],
	    True, Return[VisibleSkeleton[vlist0, ecdlist, redead4,{amat, bvec}, uvp1]]];
      On[Dot::dotsh];
      On[Part::partw];
      On[Line::list];
] 



SkelReasonableSet[dirname_String, filename_String, game_, opts___?OptionQ] := 
   Module[{norm,normgame, propcont, vertices, newext, extlist, linearity,ecdlist, eadlist, icdlist, iadlist, inedata,figure,amat,bvec,vlist0,uvp1,m0,d0,dirnew,skl,fileskloogl,tofile2},
      showskl = ShowSkeleton /. {opts} /. Options[SkelImputationSet]; 
      propcont =  WithProperContribution /. {opts} /. Options[SkelImputationSet]; 
      norm = OneNormalization[game]; 
      normgame = DefineGame[T,norm];
      {{extlist, linearity}, ecdlist, eadlist, icdlist, iadlist, inedata} = Which[propcont===False, CddGmpPlotReasonableSet[normgame],
                                                                                  True,CddGmpPlotReasonableSet[normgame, ProperContribution -> True]];
       Off[Dot::dotsh];
       Off[Part::partw];
       Off[Line::list];
       {m0,d0}= Dimensions[extlist];
       vertices = extlist;
       newext = N[vertices];
       dirnew = NewDirectory[dirname];
       amat=-Transpose[Drop[Transpose[inedata],1]];
       bvec=Transpose[inedata][[1]];
       vlist0 = Map[Drop[#, 1]&, ToExpression[newext]];
       uvp1 = circle[Pi/11, 50, 20];
       endofeadl = Take[eadlist,-4];
       redead1 = DeleteCases[#,m0+1]&/@ Drop[eadlist,-4];
       redead2 = DeleteCases[#,m0+2]&/@ redead1;
       redead3 = DeleteCases[#,m0+3]&/@ redead2;
       redead4 = DeleteCases[#,m0+4]&/@ redead3;
       skl = skel3D[uvp1,vlist0, ecdlist, redead4, amat, bvec];
       fileskloogl = ToString[filename] <> ToString[Skel] <> ToString[".oogl"];
       tofile2 = ToFileName[dirnew, fileskloogl];
       WriteOOGL[tofile2,skl];
       Which[showskl == True,
       Show[skl,Boxed->False, 
 		ViewPoint -> getMmaViewPoint[uvp1,skel3D[uvp1,vlist0, ecdlist, redead4, amat, bvec]],
 		SphericalRegion->True
 	],True, Return[VisibleSkeleton[vlist0, ecdlist, redead4,{amat, bvec}, uvp1]]];
       On[Dot::dotsh];
       On[Part::partw];
       On[Line::list];
] 


SkelLowerSet[dirname_String, filename_String, game_, opts___?OptionQ] := 
   Module[{norm,normgame, vertices, newext, extlist, linearity,ecdlist, eadlist, icdlist, iadlist, inedata,figure,amat,bvec,vlist0,uvp1,m0,d0,dirnew,skl,fileskloogl,tofile2},
      showskl = ShowSkeleton /. {opts} /. Options[SkelImputationSet]; 
      norm = OneNormalization[game]; 
      normgame = DefineGame[T,norm];
      {{extlist, linearity}, ecdlist, eadlist, icdlist, iadlist, inedata} =  CddGmpPlotLowerSet[normgame];
       Off[Dot::dotsh];
       Off[Part::partw];
       Off[Line::list];
       {m0,d0}= Dimensions[extlist];
       vertices = extlist;
       newext = N[vertices];
       dirnew = NewDirectory[dirname];
       amat=-Transpose[Drop[Transpose[inedata],1]];
       bvec=Transpose[inedata][[1]];
       vlist0 = Map[Drop[#, 1]&, ToExpression[newext]];
       uvp1 = circle[Pi/11, 50, 20];
       endofeadl = Take[eadlist,-4];
       redead1 = DeleteCases[#,m0+1]&/@ Drop[eadlist,-4];
       redead2 = DeleteCases[#,m0+2]&/@ redead1;
       redead3 = DeleteCases[#,m0+3]&/@ redead2;
       redead4 = DeleteCases[#,m0+4]&/@ redead3;
       skl = skel3D[uvp1,vlist0, ecdlist, redead4, amat, bvec];
       fileskloogl = ToString[filename] <> ToString[Skel] <> ToString[".oogl"];
       tofile2 = ToFileName[dirnew, fileskloogl];
       WriteOOGL[tofile2,skl];
       Which[showskl == True,
       Show[skl,Boxed->False, 
 		ViewPoint -> getMmaViewPoint[uvp1,skel3D[uvp1,vlist0, ecdlist, redead4, amat, bvec]],
 		SphericalRegion->True
 	],True, Return[VisibleSkeleton[vlist0, ecdlist, redead4,{amat, bvec}, uvp1]]];
       On[Dot::dotsh];
       On[Part::partw];
       On[Line::list];
] 


WriteSkelToOOGL[dirname_String, filename_String,graphicObj_]:= Module[{fileskloogl,tofile2},
       fileskloogl = ToString[filename] <> ToString[Skel] <> ToString[".oogl"];
       tofile2 = ToFileName[dirname, fileskloogl];
       WriteOOGL[tofile2,graphicObj];

]



AdjustVert[game_,vertices_]:= Module[{rl,permut,grpermut,adjvert},
    permut = Permutations[{1,0,0,0}];
    grpermut = Permutations[{v[T],0,0,0}];
    rl= MapThread[Rule,{permut,grpermut}];
    adjvert = vertices /. rl;
    adjvert
 ]


BreakCalc[dim_Integer] := (
    Print["Dimension is not 4"];
    Print["Dimension is ",dim];
    Break[]

);



(* 

  The following code stacks are borrowed from Fukuda
  cddml-PolytopeSkeleton.nb 

*)

invertpos[l_List, j_]:=
	Module[{i,pos={}},
		Do[
			If[Position[l[[i]],j]!={},AppendTo[pos,i]],
			{i,1,Length[l]}
		];
		pos
	];
dualizeIndices[ecd_List, ineLen_Integer]:=
	Map[invertpos[ecd,#]&, Range[ineLen]];




skel3D[vp_,vertices_List, ecdlist_List, eadlist_List, amat_List, bvec_List]:= 
	Graphics3D[{EdgeForm[{RGBColor[1,0,0], Thickness[0.02]}],
		VisibleSkeleton[vertices, ecdlist, eadlist, 
     	{amat, bvec}, vp]}
     ]; 
(*
facets=Map[(Part[vlist,#]) &, icdlist];
facets1=OrderFacets[facets];
solid3D:= Graphics3D[Polygon /@ facets1]; 
*)
uvp1 = circle[Pi/11, 50, 20];


(*
AnimateCore[graphic_,]:= Do[ uvp=circle[a,50,20];
	Show[graphic, 
		Boxed->False, 
 		ViewPoint -> getMmaViewPoint[uvp,skel3D[uvp]],
 		SphericalRegion->True
 	],
 	{a,Pi/11,Pi,Pi/18}
 ]
*)
DrawSkelCore[] := Show[skel3D[uvp], 
		Boxed->False, 
 		ViewPoint -> getMmaViewPoint[uvp,skel3D[uvp]],
 		SphericalRegion->True
 	]


AnimateSkelCore[]:= Do[ uvp=circle[a,50,20];
	Show[skel3D[uvp], 
		Boxed->False, 
 		ViewPoint -> getMmaViewPoint[uvp,skel3D[uvp]],
 		SphericalRegion->True
 	],
 	{a,Pi/11,Pi,Pi/18}
 ]


(* End of Fukuda code stack*)





End[]


(*
Protect[InterLockingImpReasSet]
Protect[NewDirectory]
Protect[PlotCore3D]
Protect[SaveVertices]
Protect[ShowNucleolus]
Protect[ShowKernel]
Protect[ShowShapley]
Protect[SkelCore]
Protect[SkelImputationSet]
Protect[SkelReasonableSet]
Protect[SkelStrongEpsCore]
Protect[VertexFacetsIncidences]
Protect[CheckNucEqKernel]
Protect[GenShapleyGraphPoint]
Protect[GenKernelGraphPoint]
Protect[GenKernelGraphLine]
Protect[GenNucleolusGraphPoint]
Protect[CheckVert]
Protect[GeneratePolyhedron]
Protect[WriteFileToOOGL]
Protect[SubCallStrCore]
Protect[MvSkelEpsCore]
Protect[ConstEpsList]
Protect[CheckNucEqKernel]
Protect[ConstructEadList]
Protect[ExtractCoreEl]
Protect[SubCallCore]
Protect[RecDeleteVert]
Protect[DeleteVert]
Protect[ReduceDim]
Protect[DropingSuperfluousVertexInc]
Protect[GeneratingIncidenceMatrix]
Protect[SetMatrix]
Protect[WriteSkelToOOGL]
Protect[AdjustVert]
Protect[BreakCalc]
*)



EndPackage[]
