(*  
    : Title : TuGamesView3D.m
    : Release Date : 27.07.2005      

*)
(* :Summary:	
    This package provides some graphical extensions to the package TuGames.
*)		

(* : Author : 
     Holger Ingmar Meinhardt
     Department of Economics University of Karlsruhe
     hme@vwl3.wiwi.uni-karlsruhe.de
*)
(*
  : Mathematica Version : 3.0, 4.0, 4.1, 4.2, 5.0, 5.1
*)
(* :Summary:	
    This package provides some graphical extensions to the package TuGames.
    The package requires MathGL3D, see also Requirements.
*)			
(*  :Sources:
    Mathematica Graphics by Tom Wickham-Jones, Springer-Telos, 1994.

*)

(*:Requirements: 
   
   The Mathematica package ExtendGraphics`View3D by Tom Wickham-Jones.
   Moreover, the package needs MathGL3d from Jens-Peer Kuska, cddml and some Mathematica packages by Komei Fukuda.
   Moreover qhull and geomview are required to use OGGL format for displaying the graphics and rotating raw
   3D graphics. 
   http://phong.informatik.uni-leipzig.de/~kuska/mview3d.html (MathGL3D)
   http://www.cs.mcgill.ca/~fukuda (cddml, PolytopeSkeleton.m, IOPolyhedra.m, UnfoldPolytope.m)
   http://www.geom.umn.edu/software/qhull
   http://www.geomview.org

*)


(*
BeginPackage["TuGamesView3D`"]
*)

MvCorePlot3D::usage = 
"MvCorePlot3D[dirname,filename,game,options] plots the strong epsilon-core, the core or combination of both."

MvKernelCatcher::usage = 
"MvKernelCatcher[dirname,filename,game,options] plots various kernel catcher and combinations."


(* Some options are shadowed by other definitions. *)
(*
Begin["`Private`"]
*)
(*
Needs["coop`CooperativeGames`"];
Needs["TuGames`"]
Needs["IOTuGames`"]
*)

Which[$MachineName === "xxx", Needs["View3D`"];,
        $MachineName === "xxx", Needs["View3D`"];,
        $MachineName === "xxx-e202", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx-e101", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx-e102", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx-e103", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx-e000", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx-e001", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx-e002", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx-e003", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx", Needs["ExtendGraphics`View3D`"];,
        $MachineName === "xxx", Needs["ExtendGraphics`View3D`"];,
        True, Needs["View3D`"];
       ]



<<PolytopeSkeleton.m
<<IOPolyhedra.m
<< UnfoldPolytope.m
Needs["Utilities`FilterOptions`"]

Get["MathGL3d`OpenGLViewer`"]

mvopts = Sort[Flatten[{ShowStrongEpsCore->False, ShowCore->True, CriticalValue -> {1}, OutputDirname -> False, InternalViewPoint->True, ViewKernelSol -> False, KernelCoord -> {}, ViewShapleySol -> False, ShapleyCoord -> {}, ViewNucleolusSol-> False, NucleolusCoord->{}, SaveAsEPS->False, SaveAsPOV->False,Options[MVShow3D]}]]
Options[MvCorePlot3D] = Options[mvopts]
Options[MvKernelCatcher] = Sort[Flatten[{ShowLowerSet->False, ShowUpperSet-> False, MvOutput -> True, Options[mvopts]}]]



MvCorePlot3D[dirname_String,filename_String,game_,opts___?OptionQ] := 
  Module[{crit,strCq, shcr, viewker,viewshap,viewnuc,seps,spov,skellist,norm,normgame,dimval,graph1,graph2,kersol,nucsol,shapsol,
       filopts,figlist1,figlist2,figlist3,mv1,mv2,mv3,mv4},
        crit = CriticalValue /. {opts} /. Options[MvCorePlot3D];
        strCq =  ShowStrongEpsCore /. {opts} /. Options[MvCorePlot3D];
        shcr =  ShowCore /. {opts} /. Options[MvCorePlot3D];
        viewker = ViewKernelSol  /. {opts} /. Options[MvCorePlot3D]; 
        viewshap = ViewShapleySol  /. {opts} /. Options[MvCorePlot3D]; 
        viewnuc = ViewNucleolusSol  /. {opts} /. Options[MvCorePlot3D]; 
        seps = SaveAsEPS  /. {opts} /. Options[MvCorePlot3D]; 
        spov = SaveAsPOV  /. {opts} /. Options[MvCorePlot3D];
        norm = OneNormalization[game];
        normgame = DefineGame[T,norm];
	skellist = SkelStrongEpsCore[dirname, filename, normgame, opts,
            ShowStrongEpsCore -> strCq, ShowCore -> shcr, CriticalValue -> crit,
            ViewPoint -> {1.714, 2.478, 1.540},ViewKernelSol -> viewker, ViewShapleySol-> viewshap, ViewNucleolusSol -> viewnuc, 
                  MvOutput-> True, KcOutput -> True];
        skellist = If[Length[T]==3,AdjustSkelList[skellist],skellist];
(*        dimval = Dimensions[skellist][[1]];*)
        dirnew = NewDirectory[dirname];
        graph1 = skellist[[1]]; (* Core Skeleton *)
        graph2 = skellist[[2]]; (* Strong Core Skeleton *)
        filopts = FilterOptions[MVShow3D, opts]; 
        kersol =  ShowKernel[normgame, opts, ShowSkeleton -> False];
        nucsol =  ShowNucleolus[normgame, opts, ShowSkeleton -> False];
        shapsol = ShowShapley[normgame, opts, ShowSkeleton -> False];
        figlist1 = Graphics3D[#, Axes -> False, Boxed -> False, 
                   ViewPoint -> {1.714, 2.478, 1.540},
                   LightSources -> {{{0.335180, -1.1415, -0.76458}, 
                   RGBColor[1., 0., 0.]}, {{1.5922, 0., -0.681822}, 
                   RGBColor[0., 1., 0.]}, {{0.5276, -0.82099, -1.0235144}, 
                   RGBColor[0., 0., 1.]}, {{0., 0., 1.5}, RGBColor[0.25, 0.25, 0.25]}},
                   DisplayFunction -> Identity]&/@ {graph1,graph2};
        figlist2 = Which[(viewker && viewnuc) === True,
                         Which[nucsol===kersol,KerShapFig[kersol,shapsol],True,KerNucShapFig[kersol,shapsol,nucsol]],
                          ((!viewker) && viewnuc) === True,KerShapFig[nucsol,shapsol],
                          (viewker && (!viewnuc)) === True,KerShapFig[kersol,shapsol],
                   True,KerShapFig[kersol,shapsol]];
        figlist1 = If[strCq == True, figlist1, Drop[figlist1,-1]];
        figlist3 = FlattenAt[Append[figlist1,figlist2],Length[figlist1]+1];
        Which[(viewker && viewshap) === True,
             mv1 = MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth];
             mv2 = MVShow3D[#, filopts, MVNewScene -> False, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth]
                         &/@ Drop[figlist3,1];,
             (viewker && (!viewshap)) === True,
               Which[(viewnuc && (nucsol===kersol)) === True,
                  mv1 = MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth];
                  mv2 = MVShow3D[#, filopts, MVNewScene -> False, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth]
                         &/@ Delete[Drop[figlist3,1],-1];,
             ((!viewnuc) && (nucsol===kersol)) === True,
                  mv1 = MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth];
                  mv2 = MVShow3D[#, filopts, MVNewScene -> False, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth]
                         &/@ Delete[Drop[figlist3,1],-1];,
               True, 
                  mv1 = MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth];
                  mv2 = MVShow3D[#, filopts, MVNewScene -> False, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth]  
                        &/@ Delete[Drop[figlist3,1],-2];
                ],
             ((!viewker) && viewshap) === True,
               Which[viewnuc ==False,
                  mv1 = MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth];
                  mv2 = MVShow3D[#, filopts, MVNewScene -> False, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth]  
                         &/@ Delete[Drop[figlist3,1],-2];,
               True,
                  mv1 = MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth];
                  mv2 = MVShow3D[#, filopts, MVNewScene -> False, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth]  
                        &/@ Drop[figlist3,1];
               ],
             ((!viewker) && (!viewshap)) === True,
               Which[viewnuc ==True,
                  mv1 = MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth];
                  mv2 = MVShow3D[#, filopts, MVNewScene -> False, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth]  
                         &/@ Delete[Drop[figlist3,1],-1];
                  (* FlattenAt[{mv1,mv2},2] *);,
               True, mv1 = MVShow3D[#, filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth] &/@ figlist1;
               ],
             True, MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth]];
       Which[(seps && spov)===True, Print["Saving files in EPS and POV format."]; MVPasteGraphics[]; WriteFileToEPS[dirnew,ToString[filename]];WriteFileToPOV[dirnew,ToString[filename]];,
             (seps && (!spov)) === True,Print["Saving file in EPS format."];MVPasteGraphics[];WriteFileToEPS[dirnew,ToString[filename]];,
             ((!seps) && spov) ===True, Print["Saving file POV format."];MVPasteGraphics[];WriteFileToPOV[dirnew,ToString[filename]];,  
             True,MVPasteGraphics[]];
   ]  


MvKernelCatcher[dirname_String,filename_String,game_,opts___?OptionQ] := 
  Module[{crit,strCq,viewker,viewshap,viewnuc,shups,shcr,seps,spov,norm,normgame,kersol,nucsol,shapsol,filopts,figlist1,figlist2,figlist3,mv1,mv2,mv3,mv4},
        crit = CriticalValue /. {opts} /. Options[MvKernelCatcher]; 
        strCr =  ShowStrongEpsCore /. {opts} /. Options[MvKernelCatcher]; 
        viewker = ViewKernelSol  /. {opts} /. Options[MvKernelCatcher]; 
        viewshap = ViewShapleySol  /. {opts} /. Options[MvKernelCatcher]; 
        viewnuc = ViewNucleolusSol  /. {opts} /. Options[MvKernelCatcher]; 
        shups = ShowUpperSet  /. {opts} /. Options[MvKernelCatcher];
        shcr = ShowCore /. {opts} /. Options[MvKernelCatcher];
        shlws = ShowLowerSet /. {opts} /. Options[MvKernelCatcher];
        mvout = MvOutput /. {opts} /. Options[MvKernelCatcher];
        seps = SaveAsEPS  /. {opts} /. Options[MvKernelCatcher]; 
        spov = SaveAsPOV  /. {opts} /. Options[MvKernelCatcher];
        dirnew = NewDirectory[dirname];
        reass=True; (*Aux. var. to take the reasonable set instead of the upper set in CallShowkernelCatcher[] *)
        norm = OneNormalization[game];
        normgame = DefineGame[T,norm]; 
	figlist1 = CallShowKernelCatcher[dirname,filename,game,shcr,strCr,shups,reass,shlws,crit];
        filopts = FilterOptions[MvKernelCatcher, opts]; 
        kersol =  ShowKernel[normgame, opts, ShowSkeleton -> False];
        nucsol =  Which[viewnuc===True, ShowNucleolus[normgame, opts, ShowSkeleton -> False],True,{}];
        shapsol = ShowShapley[normgame, opts, ShowSkeleton -> False];
        figlist2 = Which[(viewker && viewnuc) === True,
                         Which[nucsol===kersol,KerShapFig[kersol,shapsol],True,KerNucShapFig[kersol,shapsol,nucsol]],
                          ((!viewker) && viewnuc) === True,KerShapFig[nucsol,shapsol],
                          (viewker && (!viewnuc)) === True,KerShapFig[kersol,shapsol],
                   True,KerShapFig[kersol,shapsol]];

(*
        figlist1 = If[strCq == True, figlist1, Drop[figlist1,-1]]; 
*)
        figlist3 = FlattenAt[Append[figlist1,figlist2],Length[figlist1]+1];
        Which[(viewker && viewshap) === True,
             mv1 = MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth];
             mv2 = MVShow3D[#, filopts, MVNewScene -> False, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth]
                         &/@ Drop[figlist3,1];,
             (viewker && (!viewshap)) === True,
               Which[(viewnuc && (nucsol===kersol))===True,
                  mv1 = MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth];
                  mv2 = MVShow3D[#, filopts, MVNewScene -> False, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth] 
                        &/@ Drop[Drop[figlist3,1],-1];,
               True, 
                  mv1 = MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth];
                  mv2 = MVShow3D[#, filopts, MVNewScene -> False, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth]  
                        &/@ Drop[Drop[figlist3,1],-1];
                ],
             ((!viewker) && viewshap) === True,
               Which[viewnuc === False,
                  mv1 = MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth];
                  mv2 = MVShow3D[#, filopts, MVNewScene -> False, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth]  
                         &/@ Delete[Drop[figlist3,1],2];,
               True,
                  mv1 = MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth];
                  mv2 = MVShow3D[#, filopts, MVNewScene -> False, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth]  
                        &/@ Delete[Drop[figlist3,1],2];
               ],
             ((!viewker) && (!viewshap)) === True,
               Which[viewnuc ==True,
                  mv1 = MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth];
                  mv2 = MVShow3D[#, filopts, MVNewScene -> False, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth]  
                         &/@ Delete[Drop[figlist3,1],-1];
                  (* FlattenAt[{mv1,mv2},2] *);,
               True, 
                     mv1 = MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth];
                     mv2 = MVShow3D[#, filopts, MVNewScene -> False, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth] &/@ Drop[figlist1,1];
               ],
             True, MVShow3D[figlist3[[1]], filopts, MVNewScene -> True, MVLineTubeSize -> 0.010, MVPointSphereSize -> 1.1, MVPolygonShading -> MVSmooth]];
           Which[(seps && spov)===True, Print["Saving files in EPS and POV format."]; MVPasteGraphics[]; WriteFileToEPS[dirnew,ToString[filename]];WriteFileToPOV[dirnew,ToString[filename]];,
             (seps && (!spov)) === True,Print["Saving file in EPS format."];MVPasteGraphics[];WriteFileToEPS[dirnew,ToString[filename]];,
             ((!seps) && spov) ===True, Print["Saving file POV format."];MVPasteGraphics[];WriteFileToPOV[dirnew,ToString[filename]];,  
             True,MVPasteGraphics[]];
   ] 


CallShowKernelCatcher[dirname_String,filename_String,game_,shcr_,strCr_,shups_,reass_,shlws_,crit_]:=Module[{},
    Which[((!shcr) &&  (!shups) && (!shlws) && (!strCr))===True, (*We take here the reasonable set.*)
            ShowKernelCatcher[dirname, filename, game, ShowCore->False, ShowLowerSet -> False, ShowUpperSet -> False, ShowStrongEpsCore -> False,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],
          ((!shcr) &&  shups && (!shlws) && (!strCr))===True, (*We take here the upper set instead of the reasonable set.*)
            ShowKernelCatcher[dirname, filename, game, ShowCore->False, ShowLowerSet -> False, ShowUpperSet -> True, ShowStrongEpsCore -> False,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],
          ((!shcr) &&  (!shups) && shlws && (!strCr))===True, (*Neither upper nor reasonable set is selected. *)
            ShowKernelCatcher[dirname, filename, game, ShowCore->False, ShowLowerSet -> True, ShowUpperSet -> False, ShowStrongEpsCore -> False,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],
          ((!shcr) &&  shups && shlws && (!strCr))===True,
            ShowKernelCatcher[dirname, filename, game, ShowCore->False, ShowLowerSet -> True, ShowUpperSet -> True, ShowStrongEpsCore -> False,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],
          ((!shcr) &&  shups && shlws && strCr)===True,
            ShowKernelCatcher[dirname, filename, game, ShowCore->False, ShowLowerSet -> True, ShowUpperSet -> True, ShowStrongEpsCore -> True,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],
          ((!shcr) &&  shups && (!shlws) && strCr)===True,
            ShowKernelCatcher[dirname, filename, game, ShowCore->False, ShowLowerSet -> False, ShowUpperSet -> True, ShowStrongEpsCore -> True,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],
          ((!shcr) &&  (!shups) && shlws && strCr)===True,
            ShowKernelCatcher[dirname, filename, game, ShowCore->False, ShowLowerSet -> True, ShowUpperSet -> False, ShowStrongEpsCore -> True,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],
          ((!shcr) &&  (!shups) && (!shlws) && strCr)===True,
            ShowKernelCatcher[dirname, filename, game, ShowCore->False, ShowLowerSet -> False, ShowUpperSet -> False, ShowStrongEpsCore -> True,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],
          (shcr &&  (!shups) && (!shlws) && (!strCr))===True,
            ShowKernelCatcher[dirname, filename, game, ShowCore->True, ShowLowerSet -> False, ShowUpperSet -> False, ShowStrongEpsCore -> False,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],
          (shcr && shups && (!shlws) && (!strCr))===True,
            ShowKernelCatcher[dirname, filename, game, ShowCore->True, ShowLowerSet -> False, ShowUpperSet -> True, ShowStrongEpsCore -> False,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],
          (shcr && (!shups) && shlws && (!strCr))===True,
            ShowKernelCatcher[dirname, filename, game, ShowCore->True, ShowLowerSet -> True, ShowUpperSet -> False, ShowStrongEpsCore -> False,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],        
          (shcr && shups && shlws && (!strCr))===True,
            ShowKernelCatcher[dirname, filename, game, ShowCore->True, ShowLowerSet -> True, ShowUpperSet -> True, ShowStrongEpsCore -> False,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],        
          (shcr && shups && shlws && strCr)===True,
            ShowKernelCatcher[dirname, filename, game, ShowCore->True, ShowLowerSet -> True, ShowUpperSet -> True, ShowStrongEpsCore -> True,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],
          (shcr && shups && shlws && strCr)===True,
            ShowKernelCatcher[dirname, filename, game, ShowCore->True, ShowLowerSet -> True, ShowUpperSet -> True, ShowStrongEpsCore -> True,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],
          (shcr && shups && (!shlws) && strCr)===True,
            ShowKernelCatcher[dirname, filename, game, ShowCore->True, ShowLowerSet -> False, ShowUpperSet -> True, ShowStrongEpsCore -> True,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],
          (shcr && (!shups) && shlws && strCr)===True, (*reasonable set instead of upper set *)
            ShowKernelCatcher[dirname, filename, game, ShowCore->True, ShowLowerSet -> True, ShowUpperSet -> False, ShowStrongEpsCore -> True,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],
          (shcr && (!shups) && (!shlws) && strCr)===True, (*reasonable set instead of upper set *)
            ShowKernelCatcher[dirname, filename, game, ShowCore->True, ShowLowerSet -> False, ShowUpperSet -> False, ShowStrongEpsCore -> True,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit],
         True, ShowKernelCatcher[dirname, filename, game, ShowCore->False, ShowLowerSet -> False, ShowUpperSet -> False, ShowStrongEpsCore -> False,
               MvOutput -> True, ViewPoint -> {1.714, 2.478, 1.540},ViewNucleolus -> False, ViewKernelSol -> False, ViewShapleySol->False, CriticalValue -> crit]
    ]
]


AdjustSkelList[skelList_]:= 
      Module[{pos,newskl,newskl1,newskl2,newskl3,pothick,podash},
       pos = Position[skelList,NAN];
       newskl = Delete[skelList,Take[#,2]] &/@ pos;
       pothick = Position[newskl,Thickness[0.0035]];
       newskl1 = Delete[newskl,pothick];  
       podash = Position[newskl1,Dashing[{0.01, 0.012}]];
       newskl2 = Delete[newskl1,podash]; 
       newskl3 = FlattenAt[Append[#, {Thickness[0.0035`], Dashing[{0.01`, 0.012`}]}], -1] & /@ newskl2[[1]];
       newskl3
]


KerShapFig[kerlsg_,shaplsg_]:=
     Module[{},
      MapThread[Graphics3D[{#1, PointSize[0.025], #2}, 
      Axes -> False, Boxed -> False, ViewPoint -> {1.714, 2.478, 1.540}, 
      LightSources -> {{{0.335180, -1.1415, -0.76458}, 
      RGBColor[1., 0., 0.]}, {{1.5922, 0., -0.681822}, 
      RGBColor[0., 1., 0.]}, {{0.5276, -0.82099, -1.0235144}, 
      RGBColor[0., 0., 1.]}, {{0., 0., 1.5}, RGBColor[0.25, 0.25, 0.25]}},
      DisplayFunction -> Identity] &, {{RGBColor[1, 0, 0],RGBColor[1, 1, 0]},{kerlsg,shaplsg}}]
]

KerNucShapFig[kerlsg_,shaplsg_,nuclsg_]:=
     Module[{},
      MapThread[Graphics3D[{#1, PointSize[0.025], #2}, 
      Axes -> False, Boxed -> False, ViewPoint -> {1.714, 2.478, 1.540}, 
      LightSources -> {{{0.335180, -1.1415, -0.76458}, 
      RGBColor[1., 0., 0.]}, {{1.5922, 0., -0.681822}, 
      RGBColor[0., 1., 0.]}, {{0.5276, -0.82099, -1.0235144}, 
      RGBColor[0., 0., 1.]}, {{0., 0., 1.5}, RGBColor[0.25, 0.25, 0.25]}},
      DisplayFunction -> Identity] &, {{RGBColor[1, 0, 0],RGBColor[1, 1, 0],RGBColor[1, 1, 1]},{kerlsg,shaplsg,nuclsg}}]
]


WriteFileToEPS[dirname_String, filename_String] := 
 Module[{fileEps,tofile1},
    fileEPS = ToString[filename] <> ToString[".eps"];
    tofile1 = ToString[dirname] <> ToString["/"] <> ToString[fileEPS];
    MVWriteEPS[tofile1];
]


WriteFileToPOV[dirname_String, filename_String] := 
 Module[{filepov,tofile2},
    filePOV = ToString[filename] <> ToString[".pov"];
    tofile2 = ToString[dirname] <> ToString["/"] <> ToString[filePOV];
    MVWritePOVRay[tofile2];
]



(*
End[]
*)

(*
Protect[MvKernelCatcher]
Protect[MvCorePlot3D]
Protect[AdjustSkelList]
Protect[KerShapFig]
Protect[KerNucShapFig]
Protect[WriteFileToEPS]
Protect[WriteFileToPOV]
*)


(*
EndPackage[]
*)